使用 Python 开始你的机器学习之旅
============================================================

> 机器学习是你的简历中必需的一门技能。我们简要概括一下使用 Python 来进行机器学习的一些步骤。


![Get started with machine learning using Python](https://opensource.com/sites/default/files/styles/image-full-size/public/images/education/osdc_khan_520x292_FINAL.png?itok=lCkXsudF "Get started with machine learning using Python")

>图片来自： opensource.com

你想知道如何开始机器学习吗？在这篇文章中，我将简要概括一下使用 [Python][16] 来开始机器学习的一些步骤。Python 是一门流行的开源程序设计语言，也是在人工智能及其它相关科学领域中最常用的语言之一。机器学习简称 ML，是人工智能的一个分支，它是利用算法从数据中进行学习，然后作出预测。机器学习有助于帮助我们预测我们周围的世界。

从无人驾驶汽车到股市预测，再到在线学习，机器学习通过预测来进行自我提高的方法几乎被用在了每一个领域。由于机器学习的实际运用，目前它已经成为就业市场上最有需求的技能之一。另外，使用 Python 来开始机器学习很简单，因为有大量的在线资源，以及许多可用的 [Python 机器学习库][16]。

你需要如何开始使用 Python 进行机器学习呢？让我们来总结一下这个过程。

### 提高你的 Python 技能

由于 Python 在工业界和科学界都非常受欢迎，因此你不难找到 Python 的学习资源。如果你是一个从未接触过 Python 的新手，你可以利用在线资源，比如课程、书籍和视频来学习 Python。比如下面列举的一些资源：

*   [Python 学习之路][5]
*   [Google 开发者 Python 课程（视频）][6]
*   [Google 的 Python 课堂][7]

### 安装 Anaconda

下一步是安装 [Anacona][2]。有了 Anaconda ，你将可以开始使用 Python 来探索机器学习的世界了。Anaconda 的默认安装库包含了进行机器学习所需要的工具。

### 基本的机器学习技能

有了一些基本的 Python 编程技能，你就可以开始学习一些基本的机器学习技能了。一个实用的学习方法是学到一定技能便开始进行练习。然而，如果你想深入学习这个领域，那么你需要准备投入更多的学习时间。

一个获取技能的有效方法是在线课程。吴恩达的 Coursera [机器学习课程][20] 是一个不错的选择。其它有用的在线训练包括：

*   [Python 机器学习： Scikit-Learn 教程][8]
*   [Python 实用机器学习教程][9]

你也可以在 [LiveEdu.tv][21] 上观看机器学习视频，从而进一步了解这个领域。

### 学习更多的 Python 库

当你对 Python 和机器学习有一个好的感觉之后，可以开始学习一些[开源的 Python 库][22]。科学的 Python 库将会使完成一些简单的机器学习任务变得很简单。然而，选择什么库是完全主观的，并且在业界内许多人有很大的争论。

一些实用的 Python 库包括：

*   [Scikit-learn][10] ：一个优雅的机器学习算法库，可用于数据挖掘和数据分析任务。
*   [Tensorflow][11] ：一个易于使用的神经网络库。
*   [Theano][12] ： 一个强大的机器学习库，可以帮助你轻松的评估数学表达式。
*   [Pattern][13] ： 可以帮助你进行自然语言处理、数据挖掘以及更多的工作。
*   [Nilearn][14] ：基于 Scikit-learn，它可以帮助你进行简单快速的统计学习。

### 探索机器学习

对基本的 Python、机器学习技能和 Python 库有了一定理解之后，就可以开始探索机器学习了。接下来，尝试探索一下 Scikit-learn 库。一个不错的教程是 Jake VanderPlas 写的 [Scikit-learn 简介][23]。

然后，进入中级主题，比如 [K-均值聚类算法简介][24]、线性回归、[决策树][25]和逻辑回归。

最后，深入高级机器学习主题，比如向量机和复杂数据转换。

就像学习任何新技能一样，练习得越多，就会学得越好。你可以通过练习不同的算法，使用不同的数据集来更好的理解机器学习，并提高解决问题的整体能力。

使用 Python 进行机器学习是对你的技能的一个很好的补充，并且有大量免费和低成本的在线资源可以帮助你。你已经掌握机器学习技能了吗？可以在下面留下你的评论，或者[提交一篇文章][26]来分享你的故事。

--------------------------------------------------------------------------------

作者简介：

Michael J. Garbade 博士是旧金山 LiveEdu Inc（Livecoding.tv）的创始人兼首席执行官。Livecoding.tv 是世界上观看工程师直播编代码最先进的直播平台。你可以通过观看工程师们写网站、移动应用和游戏，来将你的技能提升到一个新的水平。MichaelJ. Garbade 博士拥有金融学博士学位，并且是一名自学成才的工程师，他喜欢 Python、Django、Sencha Touch 和视频流。

-----------

via: https://opensource.com/article/17/5/python-machine-learning-introduction

作者：[Michael J. Garbade][a]
译者：[ucasFL](https://github.com/ucasFL)
校对：[wxy](https://github.com/wxy)

本文由 [LCTT](https://github.com/LCTT/TranslateProject) 原创编译，[Linux中国](https://linux.cn/) 荣誉推出

[a]:https://opensource.com/users/drmjg
[1]:https://opensource.com/tags/python?src=programming_resource_menu
[2]:https://opensource.com/tags/javascript?src=programming_resource_menu
[3]:https://opensource.com/tags/perl?src=programming_resource_menu
[4]:https://developers.redhat.com/?intcmp=7016000000127cYAAQ&amp;amp;amp;amp;src=programming_resource_menu
[5]:https://learnpythonthehardway.org/book/
[6]:https://www.youtube.com/playlist?list=PLfZeRfzhgQzTMgwFVezQbnpc1ck0I6CQl
[7]:https://developers.google.com/edu/python/
[8]:https://www.datacamp.com/community/tutorials/machine-learning-python#gs.HfAvLRs
[9]:https://pythonprogramming.net/machine-learning-tutorial-python-introduction/
[10]:http://scikit-learn.org/stable/
[11]:https://opensource.com/article/17/2/machine-learning-projects-tensorflow-raspberry-pi
[12]:http://deeplearning.net/software/theano/
[13]:https://github.com/clips/pattern
[14]:https://github.com/nilearn/nilearn
[15]:https://opensource.com/article/17/5/python-machine-learning-introduction?rate=jgAmIV_YqoWTbnSgNjZ0EE5lyhJtzf-ukzhiMmXtfMQ
[16]:https://opensource.com/article/17/2/3-top-machine-learning-libraries-python
[17]:https://www.liveedu.tv/learn/python/
[18]:https://opensource.com/article/17/2/3-top-machine-learning-libraries-python
[19]:http://docs.continuum.io/anaconda/install
[20]:https://www.coursera.org/learn/machine-learning
[21]:https://www.liveedu.tv/
[22]:https://opensource.com/article/17/5/4-practical-python-libraries
[23]:http://nbviewer.jupyter.org/github/donnemartin/data-science-ipython-notebooks/blob/master/scikit-learn/scikit-learn-intro.ipynb
[24]:https://www.datascience.com/blog/introduction-to-k-means-clustering-algorithm-learn-data-science-tutorials
[25]:http://machinelearningmastery.com/implement-decision-tree-algorithm-scratch-python/
[26]:https://opensource.com/story
[27]:https://opensource.com/user/78291/feed
[28]:https://opensource.com/users/drmjg
